/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_ESSID_HH_
# define WIFI_ESSID_HH_

# include <iostream>
# include <string>

namespace wifi
{

  namespace frame
  {

    struct ssid_elt
    {
      enum { ssid_elt_id = 0 };

      uint8_t	elt_id;
      uint8_t	length;
      uint8_t	ssid[];
    };

  } // End of namespace wifi::frame.

  struct essid: public std::string
  {
    enum { max_len = 32 };

    essid(const frame::ssid_elt&);
    essid(const frame::ssid_elt&, size_t overrided_length);
    essid(const std::string& s = std::string ());

    std::ostream&	print(std::ostream&) const;
    std::istream&	read(std::istream&, bool throw_on_parse_error = false);


    /* Worst case : 32-character-long ESSID of non-printables.
     *
     * Will be escaped as "\xFF\xFF...(x32)", so max_len * 4 +
     * 2 printable chars.  The extra + 8 chars are because of the
     * heuristic essid::escape() uses.
     */
    std::string		escape(size_t max_len = max_len * 4 + 2 + 8) const;

  private:
    static void			skipws(std::istream&);
    static unsigned char	eat(std::istream&);
    static unsigned char	eat_xdigit(std::istream&);
    static unsigned char	eat(std::istream&, const unsigned char c);
    static unsigned		xdigit(const unsigned char c);
  };

  std::ostream&
  operator << (std::ostream& o, const essid& s);

  std::istream&
  operator >> (std::istream& i, essid& s);

} // End of namespace wifi.

# include "essid.hxx"

#endif // ! WIFI_ESSID_HH_
