/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_TIMESTAMP_HH_
# define WIFI_TIMESTAMP_HH_

# include <boost/array.hpp>

extern "C"
{
# include <inttypes.h>
}

# include <ostream>

# include <wipal/tool/hash.hh>

namespace wifi
{

  /// IEEE 802.11 %timestamp frame element.
  struct timestamp
  {
    enum { size = 8 };

    /// Construct a null %timestamp.
    timestamp();

    /// Construct a %timestamp from the low and high parts of a 64 bit word.
    timestamp(uint32_t hi, uint32_t lo);

    /// @name Comparison operators
    //@{
    bool		operator == (const timestamp&) const;
    bool		operator != (const timestamp&) const;
    bool		operator < (const timestamp&) const;
    //@}

    /// Print the %timestamp in a human readable format.
    std::ostream&	print(std::ostream&) const;

    /// Get an index suitable for tool::hash_map and tool::hash_set structures.
    size_t		hash() const;

  private:
    typedef boost::array<uint8_t, size>	digits_t;

    digits_t		digits_;
  };

  /// Dump a timestamp in a human readable format.
  std::ostream&
  operator << (std::ostream&, const timestamp&);

} // End of namespace wifi.

WP_HASH_NAMESPACE_BEGIN

  template <>
  struct hash<wifi::timestamp>
  {
    size_t	operator () (const wifi::timestamp&) const;
  };

WP_HASH_NAMESPACE_END

# include "timestamp.hxx"

#endif // ! WIFI_TIMESTAMP_HH_
